// MotionController_8134.h: interface for the CMotionController_8134 class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include "Pci8134a.h"
#include "MotionControllerBase.h"

class CMotionController_8134 : public CMotionControllerBase
{
public:
    CMotionController_8134(int index);
    virtual ~CMotionController_8134();

    BOOL Init(int& nNoOfMtr) override;
    void Close() override;

    // get the IOs status of the motor
    unsigned short GetMotionIOStatus(const CMtrProfile& mtrProfile) override;

    //  get Motion status of motor
    int GetMotionStatus(const CMtrProfile& mtrProfile) override;

    //  get Motor Pos value
    double GetMtrPos(const CMtrProfile& mtrProfile) override;

    //  get position error value
    double GetPosErr(const CMtrProfile& mtrProfile) override;

    //  check if motor is ready
    BOOL IsMotorReady(const CMtrProfile& mtrProfile) override;

    //  check if motor is on
    BOOL IsMtrOn(const CMtrProfile& mtrProfile) override;
};
